import os
import datetime

import arcpy

# Author J.Connolly USEPA

# Description: Adds the new file names to the attribute table

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.


# Variables to be set by user

# Input File Locations from step 3
InGDB = r"path\filename.gdb"

abb = "FWS"  # FWS or NMFS

# Workspace for copied and standardize files; stays static for the whole tool
ws = "path"
# Folder in workspace where outputs will be saved
name_dir = "folder name"

# in yyyymmdd received date
receivedDate = 'date'

# Field names to be added that will be used to dissolve to a single multipart polygon (Dissolve) and the join column to
# add other attributes (Filename)

JoinFieldFC = "FileName"
Dissolve = "Dissolve"


# FUNCTIONS
def fcs_in_workspace(workspace):
    # recursively checks workspaces for feature classes
    arcpy.env.workspace = workspace
    for fcs in arcpy.ListFeatureClasses():
        yield (fcs)
    for wks in arcpy.ListWorkspaces():
        for fcs in fcs_in_workspace(wks):
            yield (fcs)


def create_gdb(out_folder, out_name, outpath):
    # Create a new GDB
    if not arcpy.Exists(outpath):
        arcpy.CreateFileGDB_management(out_folder, out_name, "CURRENT")


# static Variables no user input needed unless changing structure of script

# import time
datelist = []
today_date = datetime.date.today()
datelist.append(today_date)

path_dir = ws + os.sep + str(name_dir)
outLocationCSV = path_dir + os.sep + "CSV"
OutFolderGDB = path_dir + os.sep + "GDB"

out_nameGDB = "STD_ReNm_" + str(receivedDate)

# NOTE Change this to False if you don't want GDB to be overwritten
arcpy.env.overwriteOutput = True
arcpy.env.scratchWorkspace = ""

# start clock on timing script

start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

create_gdb(path_dir, outLocationCSV, OutFolderGDB)

# for each file in inGDB adds and populates the Filename and Dissolve column
for fc in fcs_in_workspace(InGDB):
    try:
        arcpy.AddField_management(fc, str(JoinFieldFC), "TEXT", "", "", "225", "", "NULLABLE", "NON_REQUIRED", "")
        arcpy.AddField_management(fc, str(Dissolve), "TEXT", "", "", "", "5", "NULLABLE", "NON_REQUIRED", "")
    except:
        print "Failed to add dissolve fields " + str(fc)

for fc in fcs_in_workspace(InGDB):
    try:
        name = str(fc) + "_STD"
        namesimple = str(fc)
        # print namesimple
        JoinField = JoinFieldFC
        rows = arcpy.da.UpdateCursor(fc, JoinField)
        for row in rows:
            row[0] = namesimple
            rows.updateRow(row)
    except:
        print "filename not added: " + str(fc)
for fc in fcs_in_workspace(InGDB):
    try:
        rows = arcpy.da.UpdateCursor(fc, Dissolve)
        for row in rows:
            row[0] = "1"
            rows.updateRow(row)
    except:
        print "Failed to pop dissolve field:" + str(fc)

# #End clock time script
end = datetime.datetime.now()
print "End Time: " + end.ctime()

elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
