import arcpy
import os
import datetime
import pandas as pd

# Description: Checks for species with duplicate files in spatial library; this indicated that files was updated,
# or a species has two files that need to be merged. This script can be used on range or critical habitat files

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

# User input variable
# input table
# Note- Make sure all commas are removed to get accurate results!

masterlist =  r"\MasterListESA_Feb2017_20190130.csv"
# Spatial Library for location files range or critical habitat (generalized or non) \CriticalHabitat or \Range

in_folder = 'path\Generalized files\CriticalHabitat'
group_colindex = 12  # Index position of the group column in the master species list; base 0
# #########Functions


def get_group_list(master_list):
    # generates a list of sp groups from master list
    grouplist = []
    with open(master_list, 'rU') as inputFile:
        header = next(inputFile)
        for line in inputFile:
            line = line.split(',')
            group = line[group_colindex]
            grouplist.append(group)
    inputFile.close()

    unq_grps = set(grouplist)
    sorted_group = sorted(unq_grps)
    print sorted_group
    del header, grouplist
    return sorted_group


start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

masterlist_df = pd.read_csv(masterlist)  # read in master species list
sp_groups_df = masterlist_df.ix[:,group_colindex]  # get a list of species groups
sp_groups_df = sp_groups_df.drop_duplicates()  # drop duplicates from species group list
alpha_group = sorted(sp_groups_df.values.tolist())  # sorted list of species group


duplicate_files = []
for group in alpha_group:
    # for each species group, look at files in spatial library and check for duplicates
    print "\nWorking on {0}".format(group)
    group_gdb = in_folder + os.sep + str(group) + '.gdb'

    arcpy.env.workspace = group_gdb
    fclist = arcpy.ListFeatureClasses()
    entlist_fc = []

    for fc in fclist:
        entid = fc.split('_')
        entid = str(entid[1])
        counter = 0
        if entid not in entlist_fc:
            entlist_fc.append(entid)
            counter += 1
            continue
        else:
            print 'Species {0} has a duplicate file'.format(entid)
            duplicate_files.append(entid)

print 'Species {0} has a duplicate file'.format(duplicate_files)

end = datetime.datetime.now()
print "End Time: " + end.ctime()

elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
